<?php

/*
 * This file is part of Monsieur Biz' Search plugin for Sylius.
 *
 * (c) Monsieur Biz <sylius@monsieurbiz.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

declare(strict_types=1);

namespace MonsieurBiz\SyliusSearchPlugin\Model\Document;

use Elastica\ResultSet as ElasticaResultSet;
use JoliCode\Elastically\Result;
use MonsieurBiz\SyliusSearchPlugin\Adapter\ResultSetAdapter;
use Pagerfanta\Pagerfanta;
use Pagerfanta\PagerfantaInterface;

class ResultSet implements ResultSetInterface
{
    /** @var Result[] */
    private $results = [];

    private ?int $totalHits;

    private int $maxItems;

    private int $page;

    private FiltersAwareInterface $filters;

    private RangeFilterAwareInterface $priceFilter;

    private FilterAwareInterface $taxonFilter;

    private FilterAwareInterface $mainTaxonFilter;

    private Pagerfanta $pager;

    /**
     * SearchResults constructor.
     */
    public function __construct(
        int $maxItems,
        int $page,
        FiltersAwareInterface $filters,
        FilterAwareInterface $taxonFilter,
        FilterAwareInterface $mainTaxonFilter,
        RangeFilterAwareInterface $priceFilter,
        ?ElasticaResultSet $resultSet = null
    ) {
        $this->maxItems = $maxItems;
        $this->page = $page;
        $this->filters = $filters;
        $this->mainTaxonFilter = $mainTaxonFilter;
        $this->priceFilter = $priceFilter;
        $this->taxonFilter = $taxonFilter;

        // Empty result set
        if (null === $resultSet) {
            $this->totalHits = 0;
            $this->results = [];
        } else {
            /** @var Result $result */
            foreach ($resultSet as $result) {
                $this->results[] = $result->getModel();
            }
            $this->totalHits = $resultSet->getTotalHits();
        }

        $this->initPager();
    }

    /**
     * Init pager with Pager Fanta.
     */
    private function initPager(): void
    {
        $adapter = new ResultSetAdapter($this);
        $this->pager = new Pagerfanta($adapter);
        $this->pager->setMaxPerPage($this->maxItems);
        $this->pager->setCurrentPage($this->page);
    }

    /**
     * @return Result[]
     */
    public function getResults(): array
    {
        return $this->results;
    }

    public function getFilters(): array
    {
        return $this->filters->getFilters();
    }

    public function getTotalHits(): int
    {
        return $this->totalHits;
    }

    public function getPager(): PagerfantaInterface
    {
        return $this->pager;
    }

    public function getTaxonFilter(): ?FilterInterface
    {
        return $this->taxonFilter->getFilter();
    }

    public function getMainTaxonFilter(): ?FilterInterface
    {
        return $this->mainTaxonFilter->getFilter();
    }

    public function getPriceFilter(): ?RangeFilterInterface
    {
        return $this->priceFilter->getFilter();
    }
}
